/*
 * Copyright (c) 2018, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * WSO2 Inc. licenses this file to you under the Apache License,
 * Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.wso2.transport.http.netty.sender;

import io.netty.channel.ChannelHandlerContext;
import io.netty.handler.ssl.ReferenceCountedOpenSslEngine;
import io.netty.handler.ssl.ocsp.OcspClientHandler;
import org.bouncycastle.asn1.ocsp.OCSPResponseStatus;
import org.bouncycastle.cert.ocsp.BasicOCSPResp;
import org.bouncycastle.cert.ocsp.CertificateStatus;
import org.bouncycastle.cert.ocsp.OCSPResp;
import org.bouncycastle.cert.ocsp.SingleResp;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.math.BigInteger;

import javax.net.ssl.SSLSession;
import javax.security.cert.X509Certificate;

/**
 * A handler for OCSP stapling.
 */
public class OCSPStaplingHandler extends OcspClientHandler {

    private static final Logger log = LoggerFactory.getLogger(OCSPStaplingHandler.class);
    protected OCSPStaplingHandler(ReferenceCountedOpenSslEngine engine) {
        super(engine);
    }

    @Override
    protected boolean verify(ChannelHandlerContext ctx, ReferenceCountedOpenSslEngine engine)
            throws Exception {
        //Get the stapled ocsp response from the ssl engine.
        byte[] staple = engine.getOcspResponse();
        if (staple == null) {
            throw new IllegalStateException("Server didn't provide an OCSP staple!");
        }

        OCSPResp response = new OCSPResp(staple);
        if (response.getStatus() != OCSPResponseStatus.SUCCESSFUL) {
            return false;
        }

        SSLSession session = engine.getSession();
        X509Certificate[] chain = session.getPeerCertificateChain();
        BigInteger certSerial = chain[0].getSerialNumber();

        BasicOCSPResp basicResponse = (BasicOCSPResp) response.getResponseObject();
        SingleResp singleResp = basicResponse.getResponses()[0];

        CertificateStatus status = singleResp.getCertStatus();
        BigInteger ocspSerial = singleResp.getCertID().getSerialNumber();
        if (log.isDebugEnabled()) {
            String message = new StringBuilder().append("OCSP status of ").append(ctx.channel().remoteAddress())
                    .append("\n  Status: ").append(status == CertificateStatus.GOOD ? "Good" : status)
                    .append("\n  This Update: ").append(singleResp.getThisUpdate()).append("\n  Next Update: ")
                    .append(singleResp.getNextUpdate()).append("\n  Cert Serial: ").append(certSerial)
                    .append("\n  OCSP Serial: ").append(ocspSerial).toString();
            log.debug(message);
        }
        //For a OCSP response to be valid, certificate serial number should be equal to the ocsp serial number.
        return status == CertificateStatus.GOOD && certSerial.equals(ocspSerial);
    }
}

